var fs      = require('fs'),
    join    = require('path').join,
    resolve = require('path').resolve;

var instances  = {};

// modified version of utile's library
var require_lazy = function(directory) {
  var result = {},
      files = fs.readdirSync(directory);
  
  var list  = files.map(function(file) {
    if (file.substr(-3) !== '.js')
      return file;

    return file.substr(0, file.length - 3);
  })

  list.forEach(function(file) {
    Object.defineProperty(result, file, {
      get: function() {
        return result[file] = require(resolve(directory, file, 'package.json'));
      }
    });
  });

  Object.defineProperty(result, 'keys', { 
    get: function() {
      return list;
    }
  })

  return result;
};

//////////////////////////
// functions

var Wink = function(path) {

  var list = null;

  var load_metas = function() {
    if (list !== null) return list;
    list = require_lazy(path);
    return list;
  }

  var req = function(plugin_name) {
    return require(join(path, plugin_name));
  }

  // calls the given method from a plugin
  // checks whether the function has a callback defined
  // returns false if no method called, or true if called 
  var invoke = function(plugin_name, method, obj, cb) {
    var module; 

    try {
      module = req(plugin_name);
    } catch(e) {
      cb(e);
      return false;
    }

    if (typeof module[method] !== 'function') {
      cb && cb();
      return false;
    }

    var arity = module[method].length;

    if (arity == 0) {
      var res = module[method].call(obj);
      cb(null, res);
    }  else {
      module[method].call(obj, cb);
    }

    return true;
  } 

  return {
    all: function() {
      return load_metas();
    },
    get: function(plugin_name) {
      return load_metas()[plugin_name];
    },
    reload: function() {
      list = null;
      return this;
    },
    require: function(plugin_name) {
      return req(plugin_name);
    },
    invoke: function(plugin_name, method, obj, cb) { 
      return invoke(plugin_name, method, obj, cb);
    }
  }
}

//////////////////////////
// exports

exports.init = function(dir) {
  var path = resolve(dir);
  if (instances[path])
    return instances[path];

  instances[path] = new Wink(path);
  return instances[path];
}
